package net.jaekl.frank;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import java.util.Locale;

import junit.framework.Assert;

import net.jaekl.frank.octranspo.RouteMock;
import net.jaekl.frank.octranspo.RouteSummaryMock;
import net.jaekl.frank.octranspo.TripMock;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class ScheduleTest {
	static final String TITLE_PREFIX = "<HTML>\n<HEAD>\n<TITLE>";
	static final String TITLE_SUFFIX = "</TITLE>\n" + Style.CSS + "\n</HEAD>\n";
	
	
	ByteArrayOutputStream m_baos;
	PrintWriter m_pw;
	
	@Before
	public void setUp() {
		m_baos = new ByteArrayOutputStream();
		m_pw = new PrintWriter(m_baos);
	}
	
	@After
	public void tearDown() throws IOException {
		if (null != m_baos) {
			m_baos.close();
		}
	}
	
	@Test
	public void test_writeHeader() {
		Locale[] locales = { Locale.CANADA, Locale.CANADA_FRENCH, Locale.JAPAN };
		String[] titles = { "", "CARLETON", "BILLINGS BRIDGE (3204)" };
		
		for (Locale locale : locales) {
			Schedule schedule = new Schedule(locale);
			for (String title : titles) {
				String expected = TITLE_PREFIX + title + TITLE_SUFFIX;
				
				m_baos.reset();
				
				schedule.writeHeader(m_pw, title);
				m_pw.flush();
				
				String actual = m_baos.toString();
				Assert.assertEquals(expected, actual);
			}
		}
	}

	@Test
	public void test_writePage_withNoRoutes() {
		final String stopName = "RIDEAU RIDEAU";
		final int stopNo = 3009;
		
		RouteSummaryMock rsm = new RouteSummaryMock();
		rsm.mock_setDescr(stopName);
		rsm.mock_setStopNo(stopNo);
		
		Schedule schedule = new Schedule(Locale.CANADA);
		
		m_baos.reset();
		
		schedule.writePage(m_pw, rsm);
		m_pw.flush();
		
		String actual = m_baos.toString();
		
		Assert.assertTrue(actual.contains(Style.CSS));
		
		String expectedTitle = TITLE_PREFIX + "Frank: " + stopName + " (" + stopNo + ")" + TITLE_SUFFIX;
		Assert.assertTrue(actual.contains(expectedTitle));
	}
	
	@Test
	public void test_writePage_withMultipleRoutes() {
		final String stopName = "RIDEAU RIDEAU";
		final String SOUTH_KEYS = "South Keys";
		final String HURDMAN = "Hurdman"; 
		final int stopNo = 3009;
		final Date queryDate = new Date(1418357155000L);
		final double LAT1 = 45.379298;
		final double LON1 = -75.652007;
		final double LAT2 = 45.351322;
		final double LON2 = -75.652007;
		
		RouteSummaryMock rsm;
		RouteMock route;
		TripMock trip;
		
		rsm = new RouteSummaryMock();
		rsm.mock_setDescr(stopName);
		rsm.mock_setStopNo(stopNo);
		
		// First route:  Number 1
		route = new RouteMock();
		route.mock_setRouteNo(1);
		
		trip = new TripMock(queryDate);
		trip.mock_setDest(SOUTH_KEYS);
		trip.mock_setAdjTime(10);
		trip.mock_setLatitude(LAT1);
		trip.mock_setLongitude(LON1);
		trip.mock_setSpeed(0.5);
		route.mock_addTrip(trip);
		
		trip = new TripMock(queryDate);
		trip.mock_setDest(SOUTH_KEYS);
		trip.mock_setAdjTime(20);
		trip.mock_setLatitude(LAT2);
		trip.mock_setLongitude(LON2);
		trip.mock_setSpeed(22.4);
		route.mock_addTrip(trip);

		rsm.mock_addRoute(route);
		
		// Second route:  Number 4
		route = new RouteMock();
		route.mock_setRouteNo(4);
		
		trip = new TripMock(queryDate);
		trip.mock_setDest(HURDMAN);
		trip.mock_setAdjTime(17);
		route.mock_addTrip(trip);
		
		trip = new TripMock(queryDate);
		trip.mock_setDest(HURDMAN);
		trip.mock_setAdjTime(37);
		route.mock_addTrip(trip);
		
		rsm.mock_addRoute(route);
		
		// Generate the HTML 
		Schedule schedule = new Schedule(Locale.CANADA);
		
		m_baos.reset();
		
		schedule.writePage(m_pw, rsm);
		m_pw.flush();
		
		String actual = m_baos.toString();
		
		// Some rudimentary validation of the result.
		// Should really go through more permutations, and examine them more closely, here.
		Assert.assertTrue(actual.contains(Style.CSS));
		
		String expectedTitle = TITLE_PREFIX + "Frank: " + stopName + " (" + stopNo + ")" + TITLE_SUFFIX;
		Assert.assertTrue(actual.contains(expectedTitle));
		
		Assert.assertTrue(actual.contains("<TABLE ID=\"trips\">"));
		Assert.assertTrue(actual.contains("<TR><TH COLSPAN=\"5\">" + stopName + " (" + stopNo + ")</TH></TR>"));
		Assert.assertTrue(actual.contains("<TR><TH>Route</TH><TH>Destination</TH><TH>ETA</TH><TH>Remain</TH><TH>GPS Read</TH>"));
    
		Assert.assertTrue(actual.contains("<TD>1</TD>"));
		Assert.assertTrue(actual.contains("<TD>" + SOUTH_KEYS + "</TD>"));
		Assert.assertTrue(actual.contains("<TD>11:15PM</TD>"));
		Assert.assertTrue(actual.contains("<TD><SPAN ID=\"in_0\">10m</SPAN></TD>"));
		Assert.assertTrue(actual.contains("<TD><A HREF=\"" + schedule.mapUrl(LAT1, LON1) + "\">11:05:55PM</A></TD>"));
		
		Assert.assertTrue(actual.contains("<TD>1</TD>"));
		Assert.assertTrue(actual.contains("<TD>" + SOUTH_KEYS + "</TD>"));
		Assert.assertTrue(actual.contains("<TD>11:25PM"));
		Assert.assertTrue(actual.contains("<TD><SPAN ID=\"in_1\">20m</SPAN></TD>"));
		Assert.assertTrue(actual.contains("<TD><A HREF=\"" + schedule.mapUrl(LAT1, LON1) + "\">11:05:55PM</A></TD>"));
		Assert.assertTrue(actual.contains("<TD>4</TD>"));
		Assert.assertTrue(actual.contains("<TD>" + HURDMAN + "</TD>"));
		Assert.assertTrue(actual.contains("<TD><SPAN ID=\"in_2\">17m</SPAN></TD>"));
		
		Assert.assertTrue(actual.contains("<TD>4</TD>"));
		Assert.assertTrue(actual.contains("<TD>" + HURDMAN + "</TD>"));
		Assert.assertTrue(actual.contains("<TD><SPAN ID=\"in_3\">37m</SPAN></TD>"));
	}
}
